#include "channel.h"

channel::
channel() : started_(false), stopped_(false), susp_(false),
            wakeup_(false), falert_(false), recall_(false) {}

void
channel::
signal_start()
{
    scoped_lock lock(start_mtx_);
    started_ = true;
    start_sem_.notify_one();
}

void
channel::
wait_start()
{
    scoped_lock lock(start_mtx_);
    while (!started_)
        start_sem_.wait(lock);
    started_ = false;
}

void
channel::
signal_stop()
{
    scoped_lock lock(stop_mtx_);
    stopped_ = true;
    stop_sem_.notify_one();
}

void
channel::
wait_stop()
{
    scoped_lock lock(stop_mtx_);
    while (!stopped_)
        stop_sem_.wait(lock);
    stopped_ = false;
}

bool
channel::
stopped()
{
    scoped_lock lock(stop_mtx_);
    bool s = stopped_;
    stopped_ = false;
    return s;
}

void
channel::
signal_susp()
{
    scoped_lock lock(susp_mtx_);
    susp_ = true;
    susp_sem_.notify_one();
}

void
channel::
wait_susp()
{
    scoped_lock lock(susp_mtx_);
    while (susp_)
        susp_sem_.wait(lock);
    susp_ = false;
}

bool
channel::
suspended()
{
    scoped_lock lock(susp_mtx_);
    bool s = susp_;
    susp_ = false;
    return s;
}

void
channel::
clearsuspended()
{
    scoped_lock lock(susp_mtx_);
    susp_ = false;
}

void
channel::
signal_wakeup()
{
    scoped_lock lock(wakeup_mtx_);
    wakeup_ = true;
    wakeup_sem_.notify_one();
}

void
channel::
wait_wakeup()
{
    scoped_lock lock(wakeup_mtx_);
    while (!wakeup_)
        wakeup_sem_.wait(lock);
    wakeup_ = false;
}

void
channel::
signal_falert()
{
    scoped_lock lock(falert_mtx_);
    falert_ = true;
    falert_sem_.notify_one();
}

void
channel::
wait_falert()
{
    scoped_lock lock(falert_mtx_);
    while (!falert_)
        falert_sem_.wait(lock);
    falert_ = false;
}

bool
channel::
falert()
{
    scoped_lock lock(falert_mtx_);
    bool a = falert_;
    falert_ = false;
    return a;
}

void
channel::
signal_recall()
{
    scoped_lock lock(recall_mtx_);
    recall_ = true;
    recall_sem_.notify_one();
}

void
channel::
wait_recall()
{
    scoped_lock lock(recall_mtx_);
    while (!recall_)
        recall_sem_.wait(lock);
    recall_ = false;
}

bool
channel::
awoken()
{
    scoped_lock lock(wakeup_mtx_);
    bool w = wakeup_;
    wakeup_ = false;
    return w;
}

bool
channel::
recall()
{
    scoped_lock lock(recall_mtx_);
    bool r = recall_;
    recall_ = false;
    return r;
}
